"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.outputFields = void 0;
const schema_1 = require("./schema");
const complex_1 = __importDefault(require("./complex"));
const now_1 = __importDefault(require("./now"));
const object_id_1 = require("./object-id");
function default_1(schema, context = 'mongoose') {
    return outputFields(schema.fields, context);
}
exports.default = default_1;
function outputFields(fields, context) {
    const outFields = {};
    for (const [key, field1] of Object.entries(fields)) {
        const field = field1;
        const presentIn = field.presentIn;
        if (presentIn && !presentIn.includes(context))
            continue;
        if (key === '_id' || key === '__v')
            continue;
        outFields[key] = outputFieldFormat(field, context);
    }
    return outFields;
}
exports.outputFields = outputFields;
function outputFieldFormat(field, context) {
    if (isFullDeclaration(field)) {
        const typeBase = asMongooseTypeBase(field, context);
        if (!('type' in typeBase)) {
            return typeBase.plain;
        }
        const outField = { ...typeBase };
        if (field.index === true) {
            outField.index = true;
        }
        else if (field.index === 'unique') {
            outField.unique = true;
        }
        else if (field.index === 'unique-sparse') {
            outField.unique = true;
            outField.sparse = true;
        }
        outField.enum = field.enum;
        if (field.mongooseDefault !== undefined) {
            if (field.mongooseDefault === now_1.default) {
                outField.default = Date.now;
            }
            else {
                outField.default = field.mongooseDefault;
            }
        }
        if (field.mongooseRef) {
            outField.ref = field.mongooseRef;
        }
        if (field.mongooseExpires) {
            outField.expires = field.mongooseExpires;
        }
        if (field.mongoose)
            Object.assign(outField, field.mongoose);
        return omitUndefined(outField);
    }
    else {
        const typeBase = asMongooseTypeBase({ type: field }, context);
        if ('type' in typeBase)
            return typeBase;
        return typeBase.plain;
    }
}
function asMongooseTypeBase(field, context) {
    const { type } = field;
    if (type === object_id_1.ObjectId)
        return { type: require('mongoose').Schema.Types.ObjectId };
    if (type === String)
        return { type };
    if (type === Boolean)
        return { type };
    if (type === Number)
        return { type };
    if (type === Object)
        return { type };
    if (type === Date)
        return { type };
    if (complex_1.default.isComplex(type)) {
        return type.outputMongoose(context, field);
    }
    if (type instanceof Array) {
        return {
            type: type.map(subtype => {
                return outputFieldFormat(subtype, context);
            })
        };
    }
    if (schema_1.isSchema(type)) {
        const asComplex = new complex_1.default(type.fields);
        return asComplex.outputMongoose(context, { ...field, type: asComplex });
    }
    throw new Error('Unsupported type for mongoose schema ' + JSON.stringify(type));
}
function isFullDeclaration(field) {
    return !!field.type;
}
function* yieldMany(items) {
    for (const item of items)
        yield item;
}
function omitUndefined(obj) {
    const out = {};
    for (const [key, value] of Object.entries(obj)) {
        if (value !== undefined)
            out[key] = value;
    }
    return out;
}
